'' =================================================================================================
''
''   File....... loader.spin
''   Purpose.... resident loader to allow updates via SD card
''   Author..... Brian Riley, Underill Center, VT, USA
''               various parts based upon work from

''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated....
''
'' =================================================================================================


{
How to use:
1) copy this code to propeller
2) open file you want to install on the propeller
3) press F8
4) press save EEPROM file
5) save file as code.bin
6) place on uSD card or email to custoers that need updates.
7) if mass installing you can add a file called "nodelete.txt" to card
   and the file will not be deleted off the uSD card like normal. 
}


CON _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000       '80 MHz

    PAGESIZE = 32

    LO_EEPROM = $0000           ' based upon 24LC512 (64KB)
    HI_EEPROM = $8000


VAR byte buffer[PAGESIZE]
    long maincog
    long ss[50]                ' Stack for watchdog timer
OBJ
    SD : "fsrwFemto"           ' Has different mount call
    sdspi : "sdspiFemto"       ' SPIN program loader and support routines  

DAT

RunNow                  byte    "run.pgm", 0
NewLoader               byte    "loader.pgm", 0
NewFirmware             byte    "update.pgm", 0
NoDelete                byte    "nodelete.txt", 0

  
PUB start | wdcog
  maincog := cogid              ' get cog# of current cog
  
  wdcog:=cognew(wd,@ss)         ' start watchdog

  if \sd.mount(0,1,2,3) <> 0    ' access SD card (DO,Clk,DI,CS)
    sdspi.bootEEPROM(sdspi#bootAddr + $8000) ' failed

  cogstop(wdcog)                ' stop watchdog

                                ' test for file to be immediately loaded
                                ' into RAM and RUN
  if \sd.popen(@RunNow, "r") == 0
    sd.bootSDCard               ' this loads the current open file
                                ' test for file to be loaded into low
                                ' EEPROM, nominally a replacement bootloader
  \loadEEPROM(@NewLoader, LO_EEPROM)

                                ' test for file to be loaded into high
                                ' EEPROM, this will be the normal running program
  \loadEEPROM(@NewFirmware, HI_EEPROM)

  sdspi.bootEEPROM(sdspi#bootAddr + $8000) ' load from 2nd 32K of EEPROM

PRI wd
  waitcnt(cnt+clkfreq/2)        ' wait for 10th of a second

  cogstop(maincog)              ' stop hung main cog

  sdspi.bootEEPROM(sdspi#bootAddr + $8000) ' load from 2nd 32K of EEPROM
  
PRI loadEEPROM (fname, eeAdr) | a, c, d
  eeAdr += sdspi#bootAddr     ' always use boot EEPROM
  if \SD.popen(fname,"r")
     abort string("Can't open file")
  if SD.pread(@buffer,PAGESIZE) <> PAGESIZE
     abort string("Can't read program")
  if SD.writeEEPROM(eeAdr,@buffer,PAGESIZE)
     abort string("Copy EEPROM write error")
  if SD.writeWait(eeAdr)
     abort string("Copy EEPROM wait error")
  a := word[@buffer+SD#vbase]   'use actual size of program
  repeat c from PAGESIZE to a - 1 step PAGESIZE
    d := (a - c) <# PAGESIZE
    if SD.pread(@buffer,d) <> d
      abort string("Can't read program")
    if SD.writeEEPROM(eeAdr+c,@buffer,d)
      abort string("Copy EEPROM write error")
    if SD.writeWait(eeAdr+c)
      abort string("Copy EEPROM wait error")
  if \SD.pclose < 0
     abort string("Error closing file")
  if \sd.popen(@NoDelete, "r") <> 0
    \sd.popen(fname, "d")


{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
